c *** program: N_rm_h.for **** date: 18 Jul 2005 ***********************
c Remove the effect of finite-amplitude topographic mass from geoid 
c   data.
c***********************************************************************
      program N_rm_h
      implicit none
      include 'spharms.inc'
c
      integer i,j,l,m,n,nf,np
      real*8 cN(0:lmax,0:lmax),sN(0:lmax,0:lmax)
      real*8 ch(0:lmax,0:lmax),sh(0:lmax,0:lmax)
      real*8 aHsd(phmax,thmax),nHsd(phmax,thmax),ph(phmax),th(thmax)
      real*8 dph,dth,dtr
      real*8 Gc,GM,Rm,Ro0
      real*8 cc,cs,con1,con2(0:lmax),g,pi
c
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c  Universal constants:
      pi=3.141592653589793d0
      dtr=pi/180d0
      Gc=6.6732d-11           ! We only know it to 5 sigdig!?
c
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c Grid parameters:
      dth=180d0*dtr/dfloat(thmax-1)
      dph=360d0*dtr/dfloat(phmax)
      do i=1,phmax
        ph(i)=dph*dfloat(i-1)
      enddo
      do j=1,thmax
        th(j)=dth*dfloat(j-1)
      enddo
c
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c Constant parameters:
c   Rm:  Reference equatorial radius of Mars (m) (from Lemoine gravfile)
      Rm=3.397d6
c   GM:  Gravitational constant of Mars (N m^2/kg) (from Lemoine gravfile)
      GM=4.2828371901284001d13
c   g:  Gravitational acceleration at surface of Mars (m/s^2)
      g=GM/(Rm*Rm)
c   Ro0:  Crustal density (kg/m^3) (McKenzie et al. 2002, for Tharsis)
      Ro0=2.98d3
c
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c Some time-saving assignments:
      con1=4d0*pi*Gc*Ro0*Rm/g
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c Read in the spherical harmonic domain data sets:
      open(11,file='areoid_fl60.sha',status='old')
      rewind(11)
      open(12,file='topogr_fl60.sha',status='old')
      rewind(12)
c
c Read in the files:
1     read(11,'(i4,i4,e16.8,e16.8)',end=2)l,m,cc,cs
        cN(l,m)=cc
        sN(l,m)=cs
        read(12,'(i4,i4,e16.8,e16.8)')l,m,cc,cs
        ch(l,m)=cc
        sh(l,m)=cs
      goto1
2     close(11)
      close(12)
c
c Inverse transform topography to spatial domain:
      call calc_sig(ch,sh,ph,th,aHsd)
c
c Truncate the Taylor series at five terms:
      nf=1
      do n=1,5
        nf=nf*n
        do j=1,thmax
          do i=1,phmax
            nHsd(i,j)=aHsd(i,j)**n
          enddo
        enddo
c Transform to spherical harmonics:
        call gen_clms(nHsd,ph,th,ch,sh)
c Pre-calculate con2(l):
        do l=0,lmax
          np=1
          do j=2,n
            np=np*(l+4-j)
          enddo
          if(n.gt.l+3)np=0
          con2(l)=con1*dfloat(np)/(dfloat(nf*(2*l+1))*Rm**(n-1))
        enddo
c Subtract from the total geoid:
        do m=0,lmax
          do l=m,lmax
            cN(l,m)=cN(l,m)-con2(l)*ch(l,m)
            sN(l,m)=sN(l,m)-con2(l)*sh(l,m)
          enddo
        enddo
      enddo
c
c Inverse transform geoid to spatial domain:
      call calc_sig(cN,sN,ph,th,nHsd)
c
c Write to a file:
      open(11,file='geoid_less_h.dat',status='unknown')
      rewind(11)
      do j=1,thmax
        do i=1,phmax
          write(11,'(e9.3,e11.3,e20.12)')ph(i)/dtr,90d0-th(j)/dtr,
     &        nHsd(i,j)
        enddo
      enddo
c
      end
c
