c *** subroutine: gen_clms.for **** date: 16 Sep 2002 ******************
c Calculate spherical harmonic coefficients from spatial domain signal.
c
c This is an attempt to create a spherical harmonic transform
c   on a discrete grid by a fourier method.            
c   The fourier coefficients of the plm's are generated
c   directly via a recursion relation.  Because trig functions
c   are not orthogonal on the interval 0 to pi, more
c   complicated summations are used to evaluate the integral
c   of a product of two fourier series.
c
c Courtesy Sean Swenson 04 Dec 2001
c***********************************************************************
      subroutine gen_clms(sig_in,phi,th,clm,slm)
      implicit none
      include 'spharms.inc'
c
c  Arguments:
c
      real*8 sig_in(phmax,thmax),phi(phmax),th(thmax)
      real*8 clm(0:lmax,0:lmax),slm(0:lmax,0:lmax)
c
c  Local variables:
c
      integer i,j,k,l,m,n
      real*8 temp,norm,nt,mp
      real*8 deltheta,delphi,pi
      real*8 plm(0:lmax,0:lmax,0:lmax)
      real*8 dftc(0:lmax,thmax),dfts(0:lmax,thmax)
      real*8 th_cc(0:lmax,0:lmax),th_sc(0:lmax,0:lmax)

      pi=3.141592653589793d0

c--  Initialise arrays  -------------------------------------------c
      do m=1,thmax
         do l=0,lmax
            dftc(l,m)=0d0
            dfts(l,m)=0d0
         enddo
      enddo
      do m=0,lmax
         do l=0,lmax
            clm(l,m)=0d0
            slm(l,m)=0d0
            th_cc(l,m)=0d0
            th_sc(l,m)=0d0
            do k=0,lmax
               plm(l,m,k)=0d0
            enddo
         enddo
      enddo
c--  Calculate cos and sin coefficients of signal  ----------------c
      do k=1,phmax
         do m=0,lmax
            mp=dfloat(m)*phi(k)
            do i=1,thmax
               dftc(m,i)=dftc(m,i)+sig_in(k,i)*dcos(mp)
               dfts(m,i)=dfts(m,i)+sig_in(k,i)*dsin(mp)
            enddo
         enddo
      enddo

c--  Normalise fourier coefficients  ------------------------------c
      do i=1,thmax
         dftc(0,i)=dftc(0,i)/dfloat(phmax)
         dftc(lmax,i)=dftc(lmax,i)/dfloat(phmax)
         dfts(0,i)=dfts(0,i)/dfloat(phmax)
         dfts(lmax,i)=dfts(lmax,i)/dfloat(phmax)
         do m=1,lmax-1
            dftc(m,i)=dftc(m,i)*2d0/dfloat(phmax)
            dfts(m,i)=dfts(m,i)*2d0/dfloat(phmax)
         enddo
      enddo

c--  Calculate cos and sin coefficients of theta component  -------c
c--  Because the function is defined on [0,pi]  -------------------c
c--  it can be expanded in just cosine terms.  --------------------c
      do n=0,lmax 
         do i=1,thmax
            nt=th(i)*dfloat(n)
            do m=0,lmax
               th_cc(m,n)=th_cc(m,n)+2d0*dftc(m,i)*dcos(nt)
               th_sc(m,n)=th_sc(m,n)+2d0*dfts(m,i)*dcos(nt)
            enddo
         enddo
      enddo

c--  Normalise theta fourier coefficients  ------------------------c
      do m=0,lmax
         th_cc(m,0)=th_cc(m,0)/(2d0*dfloat(thmax))
         th_cc(m,lmax)=th_cc(m,lmax)/(2d0*dfloat(thmax))
         th_sc(m,0)=th_sc(m,0)/(2d0*dfloat(thmax))
         th_sc(m,lmax)=th_sc(m,lmax)/(2d0*dfloat(thmax))
         do n=1,lmax-1
            th_sc(m,n)=2d0*th_sc(m,n)/(2d0*dfloat(thmax))
            th_cc(m,n)=2d0*th_cc(m,n)/(2d0*dfloat(thmax))
         enddo
      enddo

c--  Correct normalization for incomplete coverage of the sphere  -c
      delphi=dabs(phi(2)-phi(1))
      deltheta=dabs(th(2)-th(1))
      norm=dfloat(phmax)*delphi/(2d0*pi)*dfloat(thmax)*deltheta/pi
      do n=0,lmax
         do m=0,lmax
            th_cc(m,n)=th_cc(m,n)*norm
            th_sc(m,n)=th_sc(m,n)*norm
         enddo
      enddo

c--  Calculate cos and sin coefficients of Legendre functions  ----c
c--  Expand m = even terms in a cosine series  --------------------c
c--  Expand m = odd terms in a sine series  -----------------------c

c---  Compute m = 0, l = even terms  ------------------------------c
      do l=0,lmax,2
         plm(l,0,0)=1d0
         plm(l,0,2)=dfloat(l*(l+1))*plm(l,0,0)/
     &        (dfloat(l*(l+1))-2d0)
         do j=2,l-2,2
            plm(l,0,j+2)=(2d0*dfloat(l*(l+1)-j*j)*plm(l,0,j)
     &           +dfloat((j-2)*(j-1)-l*(l+1))*plm(l,0,j-2))
     &           /dfloat(l*(l+1)-(j+2)*(j+1))
         enddo
c---  Special case for j = 0 fourier coefficient  -----------------c
         plm(l,0,0)=plm(l,0,0)/2d0

c---  Normalize overall sum to 2 for m == 0  ----------------------c
         norm=0d0
         do j=0,lmax,2
            do k=0,lmax,2
               norm=norm+plm(l,0,j)*plm(l,0,k)
     &              *(1d0/dfloat(1-j-k)+1d0/dfloat(1+j-k)
     &              +1d0/dfloat(1-j+k)+1d0/dfloat(1+j+k))/2d0
            enddo
         enddo
         norm=dsqrt(norm/2d0)
         do n=0,lmax
            plm(l,0,n)=plm(l,0,n)/norm
         enddo
      enddo

c---  Compute m = 0, l = odd terms  -------------------------------c
      do l=1,lmax,2
         plm(l,0,1)=1d0
         plm(l,0,3)=dfloat(2-l*(l+1))*plm(l,0,1)/dfloat(6-l*(l+1))
         do j=3,l-2,2
            plm(l,0,j+2)=(2d0*dfloat(l*(l+1)-j*j)*plm(l,0,j)
     &           +dfloat((j-2)*(j-1)-l*(l+1))*plm(l,0,j-2))
     &           /dfloat(l*(l+1)-(j+2)*(j+1))
         enddo

c---  Normalize overall sum to 2 for m == 0  ----------------------c
         norm=0d0
         do j=1,l,2
            do k=1,l,2
               norm=norm+plm(l,0,j)*plm(l,0,k)
     &              *(1d0/dfloat(1-j-k)+1d0/dfloat(1+j-k)
     &              +1d0/dfloat(1-j+k)+1d0/dfloat(1+j+k))/2d0
            enddo
         enddo
         norm=dsqrt(norm/2d0)
         do n=0,lmax
            plm(l,0,n)=plm(l,0,n)/norm
         enddo
      enddo

c---  Compute m = 1, l = even terms  ------------------------------c
      do l=2,lmax,2
         plm(l,1,0)=0d0
         plm(l,1,2)=1d0
         do j=2,l-2,2
            plm(l,1,j+2)=(2d0*dfloat(l*(l+1)-j*j-2)*plm(l,1,j)
     &           +dfloat((j-2)*(j-1)-l*(l+1))*plm(l,1,j-2))
     &           /dfloat(l*(l+1)-(j+2)*(j+1))
         enddo

c---  Normalize overall sum to 4 for m == 1  ----------------------c
c---  this is different than that for the cosine series  ----------c
         norm=0d0
         do j=2,l,2
            do k=2,l,2
               norm=norm+plm(l,1,j)*plm(l,1,k)
     &              *(-1d0/dfloat(1-j-k)+1d0/dfloat(1+j-k)
     &              +1d0/dfloat(1-j+k)-1d0/dfloat(1+j+k))/2d0
            enddo
         enddo
         norm=dsqrt(norm/4d0)
         do n=0,lmax
            plm(l,1,n)=plm(l,1,n)/norm
         enddo
      enddo

c---  Compute m = 1, l = odd terms  -------------------------------c
      do l=1,lmax,2
         plm(l,1,1)=1d0
         plm(l,1,3)=3d0*dfloat(l*(l+1)-2)*plm(l,1,1)/dfloat(l*(l+1)-6)
         do j=3,l-2,2
            plm(l,1,j+2)=(2d0*dfloat(l*(l+1)-j*j-2)*plm(l,1,j)
     &           +dfloat((j-2)*(j-1)-l*(l+1))*plm(l,1,j-2))
     &           /dfloat(l*(l+1)-(j+2)*(j+1))
         enddo

c---  Normalize overall sum to 4 for m == 1  ----------------------c
         norm=0d0
         do j=1,l,2
            do k=1,l,2
               norm=norm+plm(l,1,j)*plm(l,1,k)
     &              *(-1d0/dfloat(1-j-k)+1d0/dfloat(1+j-k)
     &              +1d0/dfloat(1-j+k)-1d0/dfloat(1+j+k))/2d0
            enddo
         enddo
         norm=dsqrt(norm/4d0)
         do n=0,lmax
            plm(l,1,n)=plm(l,1,n)/norm
         enddo
      enddo
c---  Compute coefficients for m > 0  -----------------------------c
c---  m = 0 terms have different normalization  -------------------c
      m=0
c---  m = 0, l = even terms  --------------------------------------c
      do l=m,lmax-2
         do j=0,l+2,2
            plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &           /dfloat(2*l+1))*plm(l,m,j)
     &           -dsqrt(dfloat((l-m+1)*(l-m+2))
     &           /dfloat(2*l+5))*plm(l+2,m,j)
     &           +dsqrt(dfloat((l-m)*(l-m-1))
     &           /dfloat(2*l+1)/2d0)*plm(l,m+2,j))
     &           /dsqrt(dfloat((l+m+4)*(l+m+3))
     &           /dfloat(2*l+5)/2d0)
         enddo
      enddo
c---  m = 0, l = odd terms  ---------------------------------------c
      do l=m+1,lmax-2
         do j=1,l+2,2
            plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &           /dfloat(2*l+1))*plm(l,m,j)
     &           -dsqrt(dfloat((l-m+1)*(l-m+2))
     &           /dfloat(2*l+5))*plm(l+2,m,j)
     &           +dsqrt(dfloat((l-m)*(l-m-1))
     &           /dfloat(2*l+1)/2d0)*plm(l,m+2,j))
     &           /dsqrt(dfloat((l+m+4)*(l+m+3))
     &           /dfloat(2*l+5)/2d0)
         enddo
      enddo

c---  m = even, > 2, l = even terms  ------------------------------c
      do m=2,lmax-2,2
         do l=m,lmax-2,2
            do j=0,l+2,2
               plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &              /dfloat(2*l+1))*plm(l,m,j)
     &              -dsqrt(dfloat((l-m+1)*(l-m+2))
     &              /dfloat(2*l+5))*plm(l+2,m,j)
     &              +dsqrt(dfloat((l-m)*(l-m-1))
     &              /dfloat(2*l+1))*plm(l,m+2,j))
     &              /dsqrt(dfloat((l+m+4)*(l+m+3))
     &              /dfloat(2*l+5))
            enddo
         enddo
c---  m = even, > 2, l = odd terms  -------------------------------c
         do l=m+1,lmax-2,2
            do j=1,l+2,2
               plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &              /dfloat(2*l+1))*plm(l,m,j)
     &              -dsqrt(dfloat((l-m+1)*(l-m+2))
     &              /dfloat(2*l+5))*plm(l+2,m,j)
     &              +dsqrt(dfloat((l-m)*(l-m-1))
     &              /dfloat(2*l+1))*plm(l,m+2,j))
     &          /dsqrt(dfloat((l+m+4)*(l+m+3))
     &              /dfloat(2*l+5))
            enddo
         enddo
      enddo

c---  m = odd, > 1, l = even terms  -------------------------------c
      do m=1,lmax-2,2
         do l=m+1,lmax-2,2
            do j=2,l+2,2
               plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &              /dfloat(2*l+1))*plm(l,m,j)
     &              -dsqrt(dfloat((l-m+1)*(l-m+2))
     &              /dfloat(2*l+5))*plm(l+2,m,j)
     &              +dsqrt(dfloat((l-m)*(l-m-1))
     &              /dfloat(2*l+1))*plm(l,m+2,j))
     &              /dsqrt(dfloat((l+m+4)*(l+m+3))
     &              /dfloat(2*l+5))
            enddo
         enddo
c---  m = odd, > 1, l = odd terms  --------------------------------c
         do l=m,lmax-2,2
            do j=1,l+2,2
               plm(l+2,m+2,j)=(dsqrt(dfloat((l+m+2)*(l+m+1))
     &              /dfloat(2*l+1))*plm(l,m,j)
     &              -dsqrt(dfloat((l-m+1)*(l-m+2))
     &              /dfloat(2*l+5))*plm(l+2,m,j)
     &              +dsqrt(dfloat((l-m)*(l-m-1))
     &              /dfloat(2*l+1))*plm(l,m+2,j))
     &              /dsqrt(dfloat((l+m+4)*(l+m+3))
     &              /dfloat(2*l+5))
            enddo
         enddo
      enddo

c--  Sum theta fourier coefficients  ----------------------------c
c--  temp is the integral of cos(n theta) cos(k theta) dcos(theta)
c--  over the interval 0 to pi.  n and k must have like parities

c---  m = even terms  -------------------------------------------c
      do m=0,lmax,2
c---  l = even terms  -------------------------------------------c
         do l=m,lmax,2
            do n=0,lmax,2
               do k=0,l,2
                  temp=0.5d0*(1d0/dfloat(1-k-n)+1d0/dfloat(1+k-n)
     &             +1d0/dfloat(1-k+n)+1d0/dfloat(1+k+n))
                  clm(l,m)=clm(l,m)+th_cc(m,n)*plm(l,m,k)*temp
                  slm(l,m)=slm(l,m)+th_sc(m,n)*plm(l,m,k)*temp
               enddo
            enddo
         enddo
c---  l = odd terms  --------------------------------------------c
         do l=m+1,lmax,2
            do n=1,lmax,2
               do k=1,l,2
                  temp=0.5d0*(1d0/dfloat(1-k-n)+1d0/dfloat(1+k-n)
     &                 +1d0/dfloat(1-k+n)+1d0/dfloat(1+k+n))
                  clm(l,m)=clm(l,m)+th_cc(m,n)*plm(l,m,k)*temp
                  slm(l,m)=slm(l,m)+th_sc(m,n)*plm(l,m,k)*temp
               enddo
            enddo
         enddo
      enddo

c---  m = odd terms  --------------------------------------------c
      do m=1,lmax,2
c---  l = even terms  (n=1 special case)  -----------------------c
         do l=m+1,lmax,2
            clm(l,m)=clm(l,m)+th_cc(m,1)*pi/4d0
     &           *(plm(l,m,2)-2d0*plm(l,m,0))
            slm(l,m)=slm(l,m)+th_sc(m,1)*pi/4d0
     &           *(plm(l,m,2)-2d0*plm(l,m,0))
            do n=3,lmax-1,2
               clm(l,m)=clm(l,m)+th_cc(m,n)*pi/4d0
     &              *(plm(l,m,n+1)-plm(l,m,n-1))
               slm(l,m)=slm(l,m)+th_sc(m,n)*pi/4d0
     &              *(plm(l,m,n+1)-plm(l,m,n-1))
            enddo
         enddo

c---  l = odd terms  (n=0, n=lmax special cases)  ---------------c
         do l=m,lmax,2
            clm(l,m)=clm(l,m)+2d0*th_cc(m,0)*pi/4d0*plm(l,m,1)
            slm(l,m)=slm(l,m)+2d0*th_sc(m,0)*pi/4d0*plm(l,m,1)
            do n=2,lmax-1,2
               clm(l,m)=clm(l,m)+th_cc(m,n)*pi/4d0
     &              *(plm(l,m,n+1)-plm(l,m,n-1))
               slm(l,m)=slm(l,m)+th_sc(m,n)*pi/4d0
     &              *(plm(l,m,n+1)-plm(l,m,n-1))
            enddo
            n=lmax
            clm(l,m)=clm(l,m)-th_cc(m,n)*pi/4d0*(plm(l,m,n-1))
            slm(l,m)=slm(l,m)-th_sc(m,n)*pi/4d0*(plm(l,m,n-1))
         enddo
      enddo

c---  Divide by Plm normalization  ------------------------------c
      do l=0,lmax
         clm(l,0)=clm(l,0)/2d0
         slm(l,0)=slm(l,0)/2d0
         do m=1,lmax
            clm(l,m)=clm(l,m)/4d0
            slm(l,m)=slm(l,m)/4d0
         enddo
      enddo

      end
